@@ -27,6 +27,12 @@
 
 #define ICAL_TIME_TYPE (icaltime_get_type ())
 
+#define ALIGNED(x)      (round (x) + 0.5)
+#define MINUTES_PER_DAY 1440
+#define MAX_MINUTES     (7 * MINUTES_PER_DAY)
+
+#define gcal_clear_datetime(dt) g_clear_pointer (dt, g_date_time_unref)
+
 typedef enum
 {
   GCAL_WINDOW_VIEW_DAY,
@@ -95,6 +101,12 @@ gint            icaltime_compare_with_current                   (const icaltimet
                                                                  const icaltimetype    *date2,
                                                                  time_t                *current_time_t);
 
+GDateTime*      get_start_of_week                               (icaltimetype          *date);
+
+GDateTime*      get_end_of_week                                 (icaltimetype          *date);
+
+gboolean        is_clock_format_24h                             (void);
+
 /* code brought from evolution */
 gsize           e_strftime_fix_am_pm                            (gchar                 *str,
                                                                  gsize                  max,
