@@ -382,8 +382,8 @@ update_active_date (GcalWindow   *window,
 
       gcal_manager_set_subscriber (window->manager, E_CAL_DATA_MODEL_SUBSCRIBER (window->year_view), range_start, range_end);
 
-      g_clear_pointer (&date_start, g_date_time_unref);
-      g_clear_pointer (&date_end, g_date_time_unref);
+      gcal_clear_datetime (&date_start);
+      gcal_clear_datetime (&date_end);
     }
 
   /* month_view */
@@ -397,8 +397,8 @@ update_active_date (GcalWindow   *window,
 
       gcal_manager_set_subscriber (window->manager, E_CAL_DATA_MODEL_SUBSCRIBER (window->month_view), range_start, range_end);
 
-      g_clear_pointer (&date_start, g_date_time_unref);
-      g_clear_pointer (&date_end, g_date_time_unref);
+      gcal_clear_datetime (&date_start);
+      gcal_clear_datetime (&date_end);
     }
 
   /* week_view */
@@ -414,17 +414,16 @@ update_active_date (GcalWindow   *window,
       !g_date_valid (&old_week) ||
       g_date_get_iso8601_week_of_year (&old_week) != g_date_get_iso8601_week_of_year (&new_week))
     {
-      date_start = g_date_time_new_local (new_date->year, new_date->month, new_date->day, 0, 0, 0);
-      date_start = g_date_time_add_days (date_start, (get_first_weekday () - g_date_time_get_day_of_week (date_start) + 7) % 7 - 7);
+      date_start = get_start_of_week (new_date);
       range_start = g_date_time_to_unix (date_start);
 
-      date_end = g_date_time_add_days (date_start, 7);
+      date_end = get_end_of_week (new_date);
       range_end = g_date_time_to_unix (date_end);
 
       gcal_manager_set_subscriber (window->manager, E_CAL_DATA_MODEL_SUBSCRIBER (window->week_view), range_start, range_end);
 
-      g_clear_pointer (&date_start, g_date_time_unref);
-      g_clear_pointer (&date_end, g_date_time_unref);
+      gcal_clear_datetime (&date_start);
+      gcal_clear_datetime (&date_end);
     }
 
   update_today_button_sensitive (window);
@@ -1131,7 +1130,6 @@ edit_dialog_closed (GtkDialog *dialog,
   event = gcal_edit_dialog_get_event (edit_dialog);
   view = GCAL_VIEW (window->views[window->active_view]);
 
-  gcal_edit_dialog_set_event (edit_dialog, NULL);
   gtk_widget_hide (GTK_WIDGET (dialog));
 
   switch (response)
@@ -1178,6 +1176,8 @@ edit_dialog_closed (GtkDialog *dialog,
       break;
 
     }
+
+  gcal_edit_dialog_set_event (edit_dialog, NULL);
 }
 
 static void
@@ -1197,12 +1197,24 @@ search_changed (GtkEditable *editable,
                 gpointer     user_data)
 {
   GcalWindow *window = GCAL_WINDOW (user_data);
+  gint search_length;
 
   if (gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (window->search_bar)))
     {
-      /* perform the search */
-      gcal_search_view_search (GCAL_SEARCH_VIEW (window->search_view),
-                               "summary", gtk_entry_get_text (GTK_ENTRY (window->search_entry)));
+      /* perform the search or hide when the field is empty */
+
+      search_length = gtk_entry_get_text_length (GTK_ENTRY (window->search_entry));
+
+      if (search_length)
+        {
+          gtk_popover_popup (GTK_POPOVER (window->search_view));
+          gcal_search_view_search (GCAL_SEARCH_VIEW (window->search_view),
+                                   "summary", gtk_entry_get_text (GTK_ENTRY (window->search_entry)));
+        }
+      else
+        {
+          gtk_popover_popdown (GTK_POPOVER (window->search_view));
+        }
     }
 }
 
