@@ -138,20 +138,26 @@ get_event_range (GcalWeekGrid *self,
 {
   GDateTime *week_start;
   GTimeSpan diff;
+  gboolean week_start_dst;
 
   if (!self->active_date)
     return;
 
   week_start = get_start_of_week (self->active_date);
+  week_start_dst = g_date_time_is_daylight_savings (week_start);
 
   if (start)
     {
       GDateTime *event_start;
+      gboolean event_start_dst;
 
       event_start = g_date_time_to_local (gcal_event_get_date_start (event));
+      event_start_dst = g_date_time_is_daylight_savings (event_start);
 
       diff = g_date_time_difference (event_start, week_start);
+
       *start = CLAMP (diff / G_TIME_SPAN_MINUTE, 0, MAX_MINUTES);
+      *start += 60 * (event_start_dst - week_start_dst);
 
       g_clear_pointer (&event_start, g_date_time_unref);
     }
@@ -160,11 +166,15 @@ get_event_range (GcalWeekGrid *self,
     {
 
       GDateTime *event_end;
+      gboolean event_end_dst;
 
       event_end = g_date_time_to_local (gcal_event_get_date_end (event));
+      event_end_dst = g_date_time_is_daylight_savings (event_end);
 
       diff = g_date_time_difference (event_end, week_start);
+
       *end = CLAMP (diff / G_TIME_SPAN_MINUTE, 0, MAX_MINUTES);
+      *end += 60 * (event_end_dst - week_start_dst);
 
       /*
        * XXX: it may happen that the event has the same start and end
@@ -439,6 +449,14 @@ get_today_column (GcalWeekGrid *self)
   return days_diff;
 }
 
+static gboolean
+on_redraw_timeout_cb (gpointer data)
+{
+  gtk_widget_queue_draw (data);
+
+  return G_SOURCE_CONTINUE;
+}
+
 static gboolean
 gcal_week_grid_draw (GtkWidget *widget,
                      cairo_t   *cr)
@@ -605,7 +623,7 @@ gcal_week_grid_draw (GtkWidget *widget,
   if (self->redraw_timeout_id == 0)
     {
       self->redraw_timeout_id = g_timeout_add_seconds (5,
-                                                       (GSourceFunc) gtk_widget_queue_draw,
+                                                       on_redraw_timeout_cb,
                                                        self);
     }
 
@@ -679,7 +697,6 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
           guint64 events_at_range;
           gint natural_height;
           gint widget_index;
-          gint min_width;
           gint offset;
           gint height;
           gint width;
@@ -695,7 +712,6 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
           widget_index = get_event_index (overlaps, data->start, data->end);
 
           /* Gtk complains about that */
-          gtk_widget_get_preferred_width (event_widget, &min_width, NULL);
           gtk_widget_get_preferred_height (event_widget, NULL, &natural_height);
 
           /* Consider the margins of the child */
@@ -715,7 +731,7 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
           /* Setup the child position and size */
           child_allocation.x = x;
           child_allocation.y = (data->start % MINUTES_PER_DAY) * minutes_height + margin.top;
-          child_allocation.width = MAX (width, min_width);
+          child_allocation.width = width;
           child_allocation.height = height;
 
           gtk_widget_size_allocate (event_widget, &child_allocation);
