@@ -845,7 +845,7 @@ update_unchanged_events (GcalWeekHeader *self,
 static void
 update_title (GcalWeekHeader *self)
 {
-  GDateTime *week_start, *week_end;
+  GDateTime *week_start, *week_end, *week_mid;
   gchar *year_label, *month_label, *week_label;
 
   if(!self->active_date)
@@ -853,6 +853,7 @@ update_title (GcalWeekHeader *self)
 
   week_start = get_start_of_week (self->active_date);
   week_end = g_date_time_add_days (week_start, 6);
+  week_mid = g_date_time_add_days (week_start, 3);
 
   if (g_date_time_get_month (week_start) == g_date_time_get_month (week_end))
     {
@@ -867,25 +868,24 @@ update_title (GcalWeekHeader *self)
 
   if (g_date_time_get_year (week_start) == g_date_time_get_year (week_end))
     {
-      week_label = g_strdup_printf (_("week %d"), g_date_time_get_week_of_year (week_start));
       year_label = g_strdup_printf ("%d", g_date_time_get_year (week_start));
     }
   else
     {
-      week_label = g_strdup_printf (_("week %d / %d"),
-                                    g_date_time_get_week_of_year (week_start),
-                                    g_date_time_get_week_of_year (week_end));
       year_label = g_strdup_printf ("%d - %d",
                                     g_date_time_get_year (week_start),
                                     g_date_time_get_year (week_end));
     }
 
+  week_label = g_strdup_printf (_("week %d"), g_date_time_get_week_of_year (week_mid));
+
   gtk_label_set_label (GTK_LABEL (self->month_label), month_label);
   gtk_label_set_label (GTK_LABEL (self->week_label), week_label);
   gtk_label_set_label (GTK_LABEL (self->year_label), year_label);
 
   g_clear_pointer (&week_start, g_date_time_unref);
   g_clear_pointer (&week_end, g_date_time_unref);
+  g_clear_pointer (&week_mid, g_date_time_unref);
   g_clear_pointer (&month_label, g_free);
   g_clear_pointer (&week_label, g_free);
   g_clear_pointer (&year_label, g_free);
@@ -984,6 +984,14 @@ on_expand_action_activated (GcalWeekHeader *self,
     header_expand (self);
 }
 
+static gboolean
+on_redraw_timeout_cb (gpointer data)
+{
+  gtk_widget_queue_draw (data);
+
+  return G_SOURCE_CONTINUE;
+}
+
 /* Drawing area content and size */
 static gdouble
 get_weekday_names_height (GtkWidget *widget)
@@ -1339,7 +1347,7 @@ gcal_week_header_draw (GtkWidget      *widget,
   if (self->redraw_timeout_id == 0)
     {
       self->redraw_timeout_id = g_timeout_add_seconds (5,
-                                                       (GSourceFunc) gtk_widget_queue_draw,
+                                                       (GSourceFunc) on_redraw_timeout_cb,
                                                        self);
     }
 
@@ -1397,6 +1405,8 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
   GcalWeekHeader *self;
   g_autoptr (GDateTime) week_start;
   g_autoptr (GDateTime) dnd_date;
+  GDateTime *start_date;
+  GDateTime *end_date;
   GtkWidget *event_widget;
   GcalEvent *event;
   gboolean ltr;
@@ -1414,30 +1424,56 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
 
   /* RTL languages swap the drop cell column */
   if (!ltr)
-    {
-      drop_cell = 6 - drop_cell;
-    }
+    drop_cell = 6 - drop_cell;
 
   event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (event_widget));
+  start_date = gcal_event_get_date_start (event);
+  end_date = gcal_event_get_date_end (event);
   week_start = get_start_of_week (self->active_date);
-  dnd_date = g_date_time_add_days (week_start, drop_cell);
 
-  /*
-   * Set the event's start and end dates. Since the event may have a
-   * NULL end date, so we have to check it here
-   */
-  gcal_event_set_all_day (event, TRUE);
-  gcal_event_set_date_start (event, dnd_date);
+  if (gcal_event_is_multiday (event) && !gcal_event_get_all_day (event))
+    {
+      /*
+       * The only case where we don't touch the timezone is for
+       * timed, multiday events.
+       */
+      dnd_date = g_date_time_new (gcal_event_get_timezone (event),
+                                  g_date_time_get_year (week_start),
+                                  g_date_time_get_month (week_start),
+                                  g_date_time_get_day_of_month (week_start) + drop_cell,
+                                  g_date_time_get_hour (start_date),
+                                  g_date_time_get_minute (start_date),
+                                  0);
+    }
+  else
+    {
+      dnd_date = g_date_time_new_utc (g_date_time_get_year (week_start),
+                                      g_date_time_get_month (week_start),
+                                      g_date_time_get_day_of_month (week_start) + drop_cell,
+                                      0, 0, 0);
 
-  if (gcal_event_get_date_end (event))
+      gcal_event_set_all_day (event, TRUE);
+    }
+
+  /* Since the event may have a NULL end date, so we have to check it here */
+  if (end_date)
     {
       g_autoptr (GDateTime) new_end;
+      GTimeSpan difference;
 
-      new_end = g_date_time_add_days (dnd_date, 1);
+      difference = g_date_time_difference (end_date, start_date);
+      difference /= G_TIME_SPAN_HOUR;
 
+      new_end = g_date_time_add_hours (dnd_date, difference);
       gcal_event_set_date_end (event, new_end);
     }
 
+  /*
+   * Set the start date ~after~ the end date, so we can compare
+   * the event's start and end dates above
+   */
+  gcal_event_set_date_start (event, dnd_date);
+
   /* Commit the changes */
   gcal_manager_update_event (self->manager, event);
 
@@ -1583,8 +1619,32 @@ gcal_week_header_add_event (GcalWeekHeader *self,
   /* Retrieve the real start and end dates */
   if (all_day)
     {
+      GDateTime *utc_week_start, *utc_week_end, *aux;
+
+      utc_week_start = g_date_time_new_utc (g_date_time_get_year (week_start),
+                                            g_date_time_get_month (week_start),
+                                            g_date_time_get_day_of_month (week_start),
+                                            0, 0, 0);
+      utc_week_end = g_date_time_new_utc (g_date_time_get_year (week_end),
+                                          g_date_time_get_month (week_end),
+                                          g_date_time_get_day_of_month (week_end),
+                                          0, 0, 0);
+
       start_date = g_date_time_ref (gcal_event_get_date_start (event));
       end_date = g_date_time_ref (gcal_event_get_date_end (event));
+
+      /*
+       * Switch the week start and end by the UTC variants, in
+       * order to correctly compare all-day events.
+       */
+      aux = week_start;
+      week_start = utc_week_start;
+      gcal_clear_datetime (&aux);
+
+      aux = week_end;
+      week_end = utc_week_end;
+      gcal_clear_datetime (&aux);
+
     }
   else
     {
@@ -1594,13 +1654,13 @@ gcal_week_header_add_event (GcalWeekHeader *self,
 
   /* Start position */
   if (datetime_compare_date (start_date, week_start) >= 0)
-    start = (g_date_time_get_day_of_week (start_date) - get_first_weekday ()) % 7;
+    start = floor (g_date_time_difference (start_date, week_start) / G_TIME_SPAN_DAY);
   else
     start = 0;
 
   /* End position */
   if (g_date_time_compare (end_date, week_end) <= 0)
-    end = (g_date_time_get_day_of_week (end_date) - get_first_weekday ()  - all_day) % 7;
+    end = floor (g_date_time_difference (end_date, week_start) / G_TIME_SPAN_DAY) - all_day;
   else
     end = 6;
 
