@@ -145,8 +145,11 @@ update_grid_scroll_position (GcalWeekView *self)
   gdouble max, page, page_increment, value;
 
   /* While the scrolled window is not mapped, we keep waiting */
-  if (!gtk_widget_get_mapped (self->scrolled_window))
-    return G_SOURCE_CONTINUE;
+  if (!gtk_widget_get_realized (self->scrolled_window) ||
+      !gtk_widget_get_mapped (self->scrolled_window))
+    {
+      return G_SOURCE_CONTINUE;
+    }
 
   now = g_date_time_new_now_local ();
   week_start = get_start_of_week (self->date);
@@ -663,6 +666,9 @@ gcal_week_view_set_first_weekday (GcalWeekView *self,
   g_return_if_fail (GCAL_IS_WEEK_VIEW (self));
 
   self->first_weekday = day_nr;
+
+  gcal_week_header_set_first_weekday (GCAL_WEEK_HEADER (self->header), self->first_weekday);
+  gcal_week_grid_set_first_weekday (GCAL_WEEK_GRID (self->week_grid), self->first_weekday);
 }
 
 /**
